/*
 *   snortconf - configuration tool for snort.conf
 *
 *   Copyright (C) 2001 ManISec, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Yee ole includes (I put this all in one file for my sanity) */
#include "include.h"


/* and here is where all the functions that check user input for validity are	*/

int is_alpha_string(char *string) {
	long int i;
	long int len = strlen(string);
	
	for(i = 0 ; i < len; ++i) {
		if(isalpha(string[i])) {
			return TRUE;
		}
	}
	return FALSE;
}
	
/* see if string == valid IP address	*/
int snc_check_ip(char *ip) {
	short int octet[3];			/* int for IP addr octets	*/
        char i1[32], i2[32], i3[32], i4[32];	/* tmp strings for parsing	*/

	/* yes, this is probably prone to buffer overflows - but tis easy
	 * and fast, and this is NOT a suid epplication. :)		*/
        if(sscanf(ip, "%[^.].%[^.].%[^.].%[^.]", i1, i2, i3, i4) != 4) {
       		return FALSE;	/* whoops, didn't parse straight	*/
	}

	/* check yee ole string for alpha char's, in which case it's not
	   a numeric component of an IP address				*/       
	if(is_alpha_string(i1) == TRUE) return FALSE;
	if(is_alpha_string(i2) == TRUE) return FALSE;
	if(is_alpha_string(i3) == TRUE) return FALSE;
	if(is_alpha_string(i4) == TRUE) return FALSE;

	/* convert them to integers for range testing			*/
	octet[0] = atoi(i1);
        octet[1] = atoi(i2);
        octet[2] = atoi(i3);
        octet[3] = atoi(i4);

	/* make sure they're within range				*/
	if(octet[0] < 0 || octet[0] > 255) return FALSE;	
	if(octet[1] < 0 || octet[1] > 255) return FALSE;	
	if(octet[2] < 0 || octet[2] > 255) return FALSE;	
	if(octet[3] < 0 || octet[3] > 255) return FALSE;	

	return TRUE;		/* WooHoo - user entered valid IP	*/
}

int snc_check_port(char *port) {
	int p;

	if(is_alpha_string(port) == TRUE) return FALSE;
	p = atoi(port);
	if(p < 0 || p > 65535) return FALSE;
	
	return TRUE;
}

int snc_check_cidr(char *ip) {
	short int mask;
	char tmp_ip[1024];
	char *ptr;

	/* first, we see if user remembered the /XX CIDR notation
	 * also good to check, since strrchr core's if search for
	 * non-existent char :)					*/	
	if(strstr(ip, "/") == NULL) return FALSE;
	
	/* well, we made it this far, let's parse it up and see	*/
	strcpy(tmp_ip, ip);		/* cp to tmp var so we 	*/
	ptr = strrchr(tmp_ip, '/');	/* don't hse real one	*/
	ptr[0] = '\0';			/* split string into IP */
	++ptr;				/* and mask		*/
	mask = atoi(ptr);

	/* if the mask isn't between 0 and 32, return FALSE	*/
	if(mask < 0 || mask > 32) return FALSE;

	/* if IP address is not valid, return FASLE		*/
	if(snc_check_ip(tmp_ip) == FALSE) return FALSE;

	/* hey, all tests passed - must be true :)		*/
	return TRUE;
}

int snc_check_validvar(char *var) {

	if(snc_regex(var, "\\$HOME_NET|\\$EXTERNAL|\\$INTERNAL|\\$EXTERNAL_NET|\\$SMTP|\\$HTTP_SERVERS|\\$SQL_SERVERS|\\$DNS_SERVERS") == FALSE) return FALSE;

	return TRUE;
}

int snc_check_cidr_var(char *var) {
	if(snc_check_cidr(var) != TRUE) {
		if(snc_check_validvar(var) != TRUE) {
			return FALSE;
			
		}
	}
	return TRUE;
}


 
int snc_check_file(char *filename) {
	struct stat *stat_st;
        int st;
	stat_st = malloc(sizeof(struct stat));
	
	st = stat(filename, stat_st);
	if(st != 0) return FALSE;
	if(S_ISREG(stat_st->st_mode) == 0) return FALSE;
	
	free(stat_st);	
	return TRUE;
}


int snc_check_dir(char *dirname) {

	struct stat *stat_st;
        int st;
	stat_st = malloc(sizeof(struct stat));
	
	st = stat(dirname, stat_st);
	if(st != 0) return FALSE;
	if(S_ISDIR(stat_st->st_mode) == 0) return FALSE;
	
	free(stat_st);	
	return TRUE;
}

