/*
 *   snortconf - configuration tool for snort.conf
 *
 *   Copyright (C) 2001 ManISec, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Yee ole includes (I put this all in one file for my sanity) */
#include "include.h"

/* various things used to simplify printing variables w/ curses
 there should really be a lot more use of these and less of the
 hand-coded madness in evidence in much of the code here
 */
void p_button(WINDOW *win, int atr, char *txt) {

        wprintw(win, "[");              /* print the opening bracket		*/
        wattron(win, atr);              /* turn on the requested attributes	*/
        wprintw(win, "%s", txt);        /* print the text of the button		*/
        wstandend(win);                 /* end the attributes			*/
        wprintw(win, "]");              /* close the bracket			*/

}

void print_optitem(WINDOW *win, char *br, char *msg) {

        wprintw(win, "  [");
        if(color == TRUE) wattron(win, COLOR_PAIR(COLOR_CYAN));
        wprintw(win, "%s", br);
        if(color == TRUE) wstandend(win);
        wprintw(win, "]");

        wprintw(win, " %s\n", msg);

}

/* an extra special help menu - a waste of code, but prettier than the default menu's 	*/
void menu_help_main() {

        int x, y, h, w;                         /* the x y cordinates, and height/width		*/
        int opt = 0;                            /* a variable in which we store the selected option	*/
        int ch = 0;                             /* where we store keystrokes	*/
        int quit = FALSE;                       /* to quit, or not to quit, this is the flag	*/
        
	WINDOW *win;                            /* a new window we're going to create	*/

        x = ((LINES / 2) - 10);                   /* put us on X cordinate mid screen - 6 (it's going to be 12 high)	*/
        w = 80;                               /* width = the number of columns we've got to work with	*/
        y = ((COLS / 2) - 40);                                  /* Y cordinate == 0 (far left)	*/
        h = 20;                                 /* height == 12	*/
        win = newwin(h, w, x, y);               /* create that puppy	*/
        keypad(win, TRUE);

        opt = 0;

        while(ch != 13) {

                wmove(win, 1,1);

		if(color == TRUE) wattron(win, COLOR_PAIR(COLOR_CYAN)|A_BOLD);
		wprintw(win, "                 Welcome to SnortConf version %s\n\n", VERSION);
		if(color == TRUE) wstandend(win);
		wprintw(win, "  At a bare minimum, you need to visit the Network Configuration menu, and\n");
		wprintw(win, "  set the value of $HOME_NET, and it is highly recommended you at least look\n");
	        wprintw(win, "  at all the options in each menu topic available.\n\n");
                
		wprintw(win, "  You may navigate the menu system with the keystrokes listed below.\n\n");
                
		print_optitem(win, "UP_KEY / U / K", "Moves selector up one line.");
                print_optitem(win, "DOWN_KEY / D / J","Moves selector down one line.");
		print_optitem(win, "ENTER / SPACE","Activate currently selected menu option");
		                        wprintw(win, "                  (Selected item noted by [");
					/* THIS CORROSPONDS WITH sel_color_set(); */
					wsel_color_set(win);
					wprintw(win, "highlighted text");
					wstandend(win);
					wprintw(win, "]).\n");
		
                print_optitem(win, "?", "Display context-sensitive help messages in sub-menus.");
		wprintw(win, "\n");
                print_optitem(win, "Q", "Quit (exit the current menu).");
                wmove(win, (h - 2), 38);

                p_button(win, COLOR_PAIR(COLOR_CYAN) | A_BOLD, "OK");
                wmove(win, 0, 0);
                box(win, ACS_VLINE, ACS_HLINE);
                touchwin(stdscr);
		refresh();
		wrefresh(win);
                
		
		ch = getch();
                quit = TRUE;

        }


        delwin(win);
        touchwin(stdscr);

}

void menu_help_var_lineset(char *line, int w) {
	long int i;
	short int dir;
	long int c;
	
	c = 0;	/* c == how many times we've been going	*/
	dir = 0;	/* which way do we read		*/
	
	for(i = 0; line[i] != '\0' ; ++i) {
		++c;
		if(line[i] == '\n') c = 0;
		if(c > (w - 8)) {
			while(c != 0) {
				++c;
				if(line[i] == '\n') {
					c = 0;
				} else if(line[i] == ' ') {
					line[i] = '\n';
					c = 0;
				} else {
					if(dir == 0) ++i;
					if(dir == 1) --i;
				}
				
				if(c > (w - 4)) dir = 1;
			}
		}
	}
}

/* a function we can feed plain ole text with "\n"'s in it to get nice output on the screen. */
void menu_help_var(char *otext) {

	int i;					/* for use in counters and such		*/
        int x, y, h, w;                         /* the x y cordinates, and height/width		*/
        int opt = 0;                            /* a variable in which we store the selected option	*/
        int ch = 0;                             /* where we store keystrokes	*/
        int quit = FALSE;                       /* to quit, or not to quit, this is the flag	*/
      
	char *text;

	WINDOW *win;                            /* a new window we're going to create	*/

        x = ((LINES / 2) - 10);                 /* put us on X cordinate mid screen - 6 (it's going to be 12 high)	*/
        w = 80;                               /* width = the number of columns we've got to work with	*/
        y = ((COLS / 2) - 40);                                  /* Y cordinate == 0 (far left)	*/
        h = 20;                                 /* height == 12	*/
        win = newwin(h, w, x, y);               /* create that puppy	*/
        keypad(win, TRUE);
	
	text = malloc((strlen(otext) + 4));
	strcpy(text, otext);
        
	opt = 0;
	
	menu_help_var_lineset(text, w);
	
	while(ch != 13) {
		if(color == TRUE) wattron(win, COLOR_PAIR(COLOR_CYAN));
                wmove(win, 1,0);
               	wprintw(win, "  "); 
		for(i = 0; text[i] != '\0'; ++i) {
			if(text[i] == '\n') {
				/* waddch(win, text[i]); */
				wprintw(win, "\n");
				wprintw(win, "  ");
			} else {
				wprintw(win, "%c", text[i]);
			}
		} 
		if(color == TRUE) wstandend(win);
		
		wmove(win, 18, ((w / 2) - 2));

                p_button(win, COLOR_PAIR(COLOR_CYAN) | A_BOLD, "OK");
                wmove(win, 0, 0);
                box(win, ACS_VLINE, ACS_HLINE);
		refresh();
		wrefresh(win);
                ch = getch();
                quit = TRUE;

        }
        free(text);
	delwin(win);
        touchwin(stdscr);
	refresh();

}
