/*
 *   snortconf - configuration tool for snort.conf
 *
 *   Copyright (C) 2001 ManISec, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Yee ole includes (I put this all in one file for my sanity) */
#include "include.h"

/* a wrapper function for quit/exit - sanity checks and such	*/
int snc_quit_exit() {
	int retval = 'q';	/* this will set the value of 'key' to 'q' which will quit the loop this
				   function SHOULD be called from :) */
	int write_check = 0;	/* how we tell later if write_config() was successful	*/
	char tmp_str[4096];
	

	/* build string pointing to appropo snort.conf file 	*/
	strcpy(tmp_str, "Write configuration to file [");
	strcat(tmp_str, cf.snort_conf);
	strcat(tmp_str, "]?");
	
	/* if user didn't set HOME_NET, bitch at said			*/
	if(var.home_net[0] == '\0') {
		if(do_popup("$HOME_NET not set! Are you sure?", YNN) == TRUE) {
			/* since they want to go on anyway, go ahead and let them		*/
			if(snc_check_file(cf.snort_conf) == TRUE)  {
				if(do_popup(tmp_str, YN) == TRUE) {
					if(do_popup("Overwrite existing config?", YNN) == TRUE) {
					write_check = snc_write_config();
					}
				}
			} else {
				if(do_popup(tmp_str, YN) == TRUE) {
					write_check = snc_write_config();
				}
			}
		} else {
			/* they said NO! (smart user) set retval to something besides 'q'uit and exit	*/
			retval = 'Z';
		}
	} else {
		/* make user admit they're pretty sure they want the deed done			*/
		if(do_popup(tmp_str, YN) == TRUE) {
			if(snc_check_file(cf.snort_conf) == TRUE) {
				if(do_popup("Overwrite existing config?", YNN) == TRUE) {
					write_check = snc_write_config();
				}
			} else {
				if(do_popup(tmp_str, YN) == TRUE) {
					write_check = snc_write_config();
				}
			}
		}
	}
			       	
	/* in future, we write better error checks and this will be something other than "if bad" */
	if(write_check != 0) do_popup("Unknown error writing file /etc/snort/snort.conf!", HIT_OK);;

	return retval;	/* tell calling function what we decided in here		*/
}

int seq_quick_config() {

	short int q = FALSE;
	
	while(q != TRUE) {
	
		 menu_help_var("Enter the value of your $HOME_NET (ie: your local network).  This must be done using CIDR notation, which is in the format of X.X.X.X/BITMASK. (Where X.X.X.X = and IP address, and /BITMASK = the number of bits in the netmask.\n\nFor reference, see the following examples:\n/24 = Class C (255.255.255.0 in your network config)\n/16 = Class B (255.255.0.0 in your network config).\n/27 1/8th of a Class C 32 hosts\n/23 2 Class C 512 hosts\n/32 Single IP address.\n\nFor example, 192.168.1.0/24 would be the 192.168.1.* class C network.  For further reference on CIDR notation please see the Snort documentation or http://www.rfc-editor.org/rfcsearch.html");
 
		strcpy(var.home_net, popup_getval("Enter the value for $HOME_NET (ie: 192.168.1.0/24): "));
		if(snc_check_cidr(var.home_net) == FALSE) {
			do_popup("ERROR: Invalid CIDR notation!", PERR);
			var.home_net[0] = '\0';
			q = FALSE;
		} else {
			q = TRUE;
		}
	}

	
	if(var.home_net[0] != '\0') {
		if(var.external_net[0] == '\0') strcpy(var.external_net, "!$HOME_NET");
		if(var.smtp_servers[0] == '\0') strcpy(var.smtp_servers, "$HOME_NET");
		if(var.http_servers[0] == '\0') strcpy(var.http_servers, "$HOME_NET");
		if(var.sql_servers[0] == '\0') strcpy(var.sql_servers, "$HOME_NET");
		if(var.dns_servers[0] == '\0') strcpy(var.dns_servers, "$HOME_NET");
	}
	

	touchwin(stdscr);
	refresh();

	if(do_popup("Would you like to do any advanced configuration?", YNN) == FALSE) {
		return FALSE;
	} else {
		return TRUE;
	}

}

int snc_main_menu() {
 	int key = 'Z';		/* where we store keystrokes from the user 	*/
	short int sel = 0;	/* start with menu item 0 selected by default 	*/
	short int i;		/* used in yee ole for() loops and such		*/
	
	/* This is the list of all the menu items we've got in the main menu */
	char *menu_item[] = { 
		"Network Configuration          ", 
		"General Option Config          ",
		"Output Plugin Config           ",
		"Preprocessor Plugin Config     ",
		"Ruleset Selection              ",
		"SnortConf Configuration        ",
		"Logfile View (via LogTool)     \n",
		"[Q]uit and exit (asks to save) ",
		NULL 
	};
	
	/* make sure user's terminal is big enough to avoid core-dumps	*/
	if(COLS < 80 || LINES < 24) {
		clear();
		if(color == TRUE) attron(COLOR_PAIR(COLOR_RED)|A_BOLD);
		else attron(A_REVERSE);
		move(0, ((COLS / 2) - 6));
		printw("FATAL ERROR:\n\n");
		standend();
		printw("Your current terminal size is %ix%i.\n\n", COLS, LINES);
		printw("You must have a terminal size of\n");
	        printw("at least 80x24 (80 columns/24 lines)\n");
		printw("for SnortConf to operate properly.\n\n");
		printw("Please maximize your terminal window,\n");
		printw("or otherwise increase it's size.\n\n");
		printw("Press any key to exit...\n");
		getch();
		endwin();
		
		return 0;
	}

	if(cf.debug == TRUE) {
		do_popup("Debug mode enabled on STDERR", HIT_OK);
	}

	/* this code will one day be yee ole "quick config" option	*/
	if(cf.quick == TRUE) {
		if(seq_quick_config() == FALSE) {
			snc_quit_exit();
			return 0;
		}
	} 
	
	/* intro user to program if they didn't already do quick-config mode	*/
	if(cf.quick != TRUE) {
		menu_help_main();
	}
	
	/* until the user hits 'q' to quit, keep going */
	while(key != 'q') {
			
		clear();		/* clear yee ole screen		*/
		print_title_bar();	/* put yee ole title bar up top	*/
	
		/* print yee ole menu itmes to screen	*/
		for(i = 0; menu_item[i] != NULL; ++i) {
			printw("   ");				/* spaces in front to make look nice	*/
			if(i == sel) sel_color_set();	/* if selected, reverse video so user 
								   can see that it's selected		*/
			printw("%s\n", menu_item[i]);		/* print out the ole menu item		*/
			if(i == sel) standend();		/* if video was reversed above, undo 
							 	   before we do next iteneration of for	*/
		}

		refresh();					/* refresh screen for good measure	*/
		key = getch();					/* snag user key			*/
		key = tolower(key);				/* make it lowercase 			*/
								/* note, leave code 2 step for clarity	*/
		/* let us see what key, and act accordingly	*/
		switch(key) {
			case KEY_DOWN:	/* user moved down the list			*/
			case 'd':	/* if not at end, ++increment selected item	*/
			case 'j':
			case 'n':
				if(menu_item[sel + 1] != NULL) ++sel;
				break;
			case KEY_UP:	/* user moved up the list			*/
			case 'u':	/* if user not at the top of list, --increment	*/
			case 'k':
			case 'p':
				if(sel > 0) --sel;
				break;
			case 13:	/* yee ole user selected something		*/
			case ' ':
				/* depending on what item was selected, call appropo function	*/
				if(sel == 0) network_config_menu();
				if(sel == 1) general_config_menu();
				if(sel == 2) output_config_menu();
				if(sel == 3) preproc_config_menu();
				if(sel == 4) ruleset_select_menu();
				if(sel == 5) do_popup("This menu option does nothing as of yet", HIT_OK);
				if(sel == 6) do_popup("This menu option does nothing as of yet", HIT_OK);
				if(sel == 7) key = snc_quit_exit();	/* called function will set 
									   wether or not we really should
									   quit at this point depending 
									   on what happens/user input	*/
				break;
			case '?':
				menu_help_main();
				break;
			case 'q':
				key = snc_quit_exit();
				
		}
	}

	endwin();	/* make yee ole stdscr go away before exit (makes ugly mess of term if no :)	*/
	return 0;	/* get the heck out of here (someday error checking may mean return !0)		*/
}			/* all done - last bracket. :)							*/
